# coding: utf-8
from importlib import import_module
from inspect import currentframe

import django


def is_in_migration_command():
    u"""Возвращает True, если код выполняется в рамках миграций South/Django.

    :rtype: bool
    """
    from django.core.management import ManagementUtility

    def is_in_command(command):
        frame = currentframe()
        while frame:
            if 'self' in frame.f_locals:
                self_object = frame.f_locals['self']
                if isinstance(self_object, command):
                    return True

                elif isinstance(self_object, ManagementUtility):
                    # Срабатывает при использовании функции в AppConfig
                    if 'subcommand' in frame.f_locals:
                        subcommand = frame.f_locals['subcommand']
                        return subcommand == 'migrate'

            frame = frame.f_back

    if django.VERSION < (1, 7):
        commands = (
            ('south.management.commands', 'migrate'),
        )
    else:
        commands = (
            ('django.core.management.commands', 'migrate'),
            ('django.core.management.commands', 'makemigrations'),
            ('django.core.management.commands', 'sqlmigrate'),
            ('django.core.management.commands', 'showmigrations'),
        )

    for package, module_name in commands:
        try:
            module = import_module(module_name, package)
        except ImportError:
            continue

        if is_in_command(module.Command):
            return True

    return False
